package de.tu_darmstadt.tk.SmartHomeNetworkSim.control;

import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Connection;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Link;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Model;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Port;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.SmartDevice;

/**
 * Controller which allows interaction with the Model and Simulation
 *
 * @author Andreas T. Meyer-Berg
 */
public class Controller {
	/**
	 * {@link Model} which stores the smart home model
	 */
	Model model;

	/**
	 * Create a new Controller, which controls the given model and allows
	 * manipulation of this model
	 * 
	 * @param model
	 *            model, that should be controlled
	 */
	public Controller(Model model) {
		this.model = model;
	}

	/**
	 * Moves the SmartDevice device to the specified location, if it exists
	 * 
	 * @param device
	 *            the device to move
	 * @param x
	 *            new x position
	 * @param y
	 *            new y position
	 * @param z
	 *            new z position
	 */
	public void moveSmartDevice(SmartDevice device, int x, int y, int z) {
		device.setX(x);
		device.setY(y);
		device.setZ(z);

	}

	/**
	 * Changes the dimension of the model, without scaling the SmartDevice
	 * positions
	 * 
	 * @param width
	 *            the new width to set
	 * @param height
	 *            the new height to set
	 * @param depth
	 *            the new depth to set
	 */
	public void setDimension(int width, int height, int depth) {
		setDimension(width, height, depth, false);
	}

	/**
	 * Changes the dimension of the model and moves SmartDevices to new relative
	 * positions if {@code stretchModel} is true
	 * 
	 * @param width
	 *            the new width to set
	 * @param height
	 *            the new height to set
	 * @param depth
	 *            the new depth to set
	 * @param stretchModel
	 *            true, if smartDevice positions should be stretched
	 */
	public void setDimension(int width, int height, int depth, boolean stretchModel) {
		// Don't allow to small Models
		if (width < 400)
			width = 400;
		if (height < 400)
			height = 400;
		if (depth < 400)
			depth = 400;

		if (stretchModel) {
			/**
			 * Factor that changes the x position. Example: 1 -> positions stay
			 * the same 0.5 -> x position is halved 2 -> x position is doubled
			 */
			double width_factor = ((double) width) / ((double) model.getWidth());
			/**
			 * Factor that changes the y position. Example: 1 -> positions stay
			 * the same 0.5 -> y position is halved 2 -> y position is doubled
			 */
			double height_factor = ((double) height) / ((double) model.getHeight());
			/**
			 * Factor that changes the z position. Example: 1 -> positions stay
			 * the same 0.5 -> z position is halved 2 -> z position is doubled
			 */
			double depth_factor = ((double) depth) / ((double) model.getDepth());
			// Update all device positions
			for (SmartDevice d : model.getDevices()) {
				if (width_factor != 1.0)
					d.setX(scalePos(d.getX(), width_factor, width));
				if (height_factor != 1.0)
					d.setY(scalePos(d.getY(), height_factor, height));
				if (depth_factor != 1.0)
					d.setZ(scalePos(d.getZ(), depth_factor, depth));
			}
		}
		model.setWidth(width);
		model.setHeight(height);
		model.setDepth(depth);
	}

	/**
	 * Calculates the new scaled position, which should be in Bounds:
	 * {@code radius < newPosition <
	 * (upperBound - radius)}
	 * 
	 * @param oldPosition
	 *            previous position that should be scaled
	 * @param factor
	 *            how much it is stretched/shrunk
	 * @param upperBound
	 *            upper bound of the frame, which should not be exceeded
	 * @return new position that was calculated
	 */
	private int scalePos(int oldPosition, double factor, int upperBound) {
		/**
		 * New Position that should be validated and returned
		 */
		int newPosition = (int) Math.round(factor * oldPosition);
		// Update if it moves out of the frame
		if (newPosition < model.getDevice_visualization_radius())
			newPosition = model.getDevice_visualization_radius();
		if (newPosition >= upperBound - model.getDevice_visualization_radius())
			newPosition = upperBound - model.getDevice_visualization_radius();

		return newPosition;
	}

	/**
	 * Adds SmartDevice to the Model and verifies that it is inside the model
	 * bounds
	 * 
	 * @param sd
	 *            SmartDevice which should be added to the model
	 */
	public void addSmartDevice(SmartDevice sd) {
		if (model.getDevices().contains(sd))
			return;
		model.addDevices(sd);
		// validate Position
		sd.setX(scalePos(sd.getX(), 1.0, model.getWidth()));
		sd.setY(scalePos(sd.getY(), 1.0, model.getHeight()));
		sd.setZ(scalePos(sd.getZ(), 1.0, model.getDepth()));
	}

	/**
	 * Creates a new SmartDevice at the given Position. The Device is moved
	 * into the model bounds, if the position is outside the bounds
	 * 
	 * @param name
	 *            name of the smartDevice
	 * @param x_pos
	 *            position on the x-Axis
	 * @param y_pos
	 *            position on the y-Axis
	 * @param z_pos
	 *            position on the z-Axis
	 */
	public void createSmartDevice(String name, int x_pos, int y_pos, int z_pos) {
		SmartDevice sd = new SmartDevice(name);
		sd.setX(scalePos(x_pos, 1.0, model.getWidth()));
		sd.setY(scalePos(y_pos, 1.0, model.getHeight()));
		sd.setZ(scalePos(z_pos, 1.0, model.getDepth()));
		model.addDevices(sd);
	}

	/**
	 * Deletes the given SmartDevice toDelete, removes it from its links and
	 * connections.
	 * 
	 * @param toDelete
	 *            the SmartDevice to delete
	 */
	public void deleteSmartDevice(SmartDevice toDelete) {
		if (toDelete == null)
			return;
		// Remove from Links
		for (Link link : toDelete.getLinks())
			removeSmartDeviceFromLink(toDelete, link);
		// Remove Links from Device
		toDelete.getLinks().clear();

		// Delete Connections
		for (Port p : toDelete.getPorts()) {
			//Skip ports that are not connected
			if(p.getConnection()==null)continue;
			Connection c = p.getConnection();
			c.removeSmartDevice(p);
			if (c.getStatus()==Connection.FINISHED||c.getStatus()==Connection.TERMINATED) {
				//if connection terminated - remove rest of ports
				for (Port sd : c.getParticipants()) {
					if (sd != null && sd != p)
						sd.getOwner().removePort(sd);
				}
			}
		}
		//Remove all ports from the device
		toDelete.getPorts().clear();
		model.getDevices().remove(toDelete);
	}

	/**
	 * Removes the SmartDevice from the given Link
	 * 
	 * @param toRemove the Device that should be removed from the link
	 * @param link the Link, which contains the SmartDevice
	 */
	public void removeSmartDeviceFromLink(SmartDevice toRemove, Link link) {
		link.removeDevice(toRemove);
	}
}
