package de.tu_darmstadt.tk.SmartHomeNetworkSim.core;

import java.util.Collection;

/**
 * Protocol according to which the packages are created. Can contain different
 * Roles and multiple participating SmartDevice(Port)s.
 * 
 * @author Andreas T. Meyer-Berg
 */
public interface Protocol {
	
	/**
	 * Generates the next packet
	 * 
	 * @param port SmartDevice(Port) which sends the package
	 * @param timestep
	 *            Time the package should be sent, in System.currentTimeMillis
	 * @param packetLost True if the packet was lost
	 * @return next Packet, which was sent
	 */
	public Collection<Packet> generateNextPakets(Port port, long timestep, boolean packetLost);

	/**
	 * Returns the number of different roles the participating SmartDevice(Port)s
	 * could have
	 * 
	 * @return number of different roles
	 */
	public int getNumberOfRoles();

	/**
	 * Returns the textual representation of the different Roles. The Array
	 * should contain {@code NumberOfRoles} Strings. {@code Array[role]} should be
	 * a human-readable String representation of the Role at its position
	 * {@code role}.
	 * 
	 * @return String representations of the roles
	 */
	public String[] getRoles();

	/**
	 * Returns all SmartDevice(Port)s of the given Role. Returns {@code null}, if the role
	 * number was invalid.
	 * 
	 * @param role
	 *            Position of the role in {@code getNumberOfRoles}
	 * @return SmartDevices of Role with index {@code role}
	 */
	public Collection<Port> getDevicesWithRole(int role);

	/**
	 * Adds a new SmartDevice to the role, returns {@code true} if it was
	 * assigned successfully, {@code false} if it wasn't. (Either invalid role
	 * number or maximum number of devices for the role reached)
	 * 
	 * @param device
	 *            SmartDevice(Port) that should be assigned to the given role
	 * @param role
	 *            Position of the role in {@code getNumberOfRoles}
	 * @return true, if the SmartDevice(Port) was added
	 */
	public boolean addDeviceOfRole(Port device, int role);

	/**
	 * Remove a SmartDevice(Port) from this Protocol
	 * 
	 * @param device
	 *            device that should be removed
	 */
	public void removeDevice(Port device);

}
